<?php
$path = '..';
$calendar_path = "php/calendar/";
$module = 'calendar';
$obminclude = getenv('OBM_INCLUDE_VAR');

error_reporting(E_ALL & !E_WARNING);

define('OK',0);
define('ERR_CONTEXT',1);
define('ERR_INVALID_ARGUMENT',2);
define('ERR_TO_MANY_ARGUMENT',3);
define('ERR_INVALID_COMMAND',4);
define('ERR_UNKNOWN_USER',5);
define('ERR_IMPORT_ICS',6);
define('ERR_FILE_UNREADABLE',7);
define('MAX_OPTION','M');
define('PAGE_OPTION','p');
define('RESOURCE_OPTION','r');
define('BACKUPDIR', getenv('BACKUPDIR'));

//define('BACKUPDIR', '/var/www/obm-calame/backup');

define('NOT_FINISHED_MARKER', 8);

/*
 * USAGE : php [OPTIONS] calendar_exec.php [-u user -d domain] [-m mode] [-f file]command
 * 
 * Commands :
 * reset : Reset user calendar
 * import : import an ICS into user calendar
 * export : export user calendar as an ICS
 * export_resources : export resources calendar as an ICS
 *
 * Options :
 * -u => user login (mandatory)
 * -d => user domain name (mandatory)
 * -t => date for delete event, form example: 09/10/2009 (For export and reset )
 * -f => file to import (Only for import )
 * -m => Mode, default is interactive. Possible value : backup 
 * -max => max imported users for fragmented exportation
 * -p => page number for fragmented exportation
 *  in "Backup mode", store ICS into a file not into default output for all users
 *  (Only for export)
 */

function exit_with_status ($status_code_str) {
  echo "exit with $status_code_str=". constant($status_code_str)." returned\n";
  switch (constant($status_code_str)) {
    case ERR_CONTEXT :
    case ERR_INVALID_ARGUMENT :
    case ERR_TO_MANY_ARGUMENT :
      echo <<<HELP
/*
 * USAGE : calendar_exec.php [-u user -d domain] [-m mode] [-f file] command
 * 
 * Commands :
 *   reset : Reset user calendar
 *   import : import an ICS into user calendar
 *   export : export user calendar as an ICS
 *   export_resources : export resources calendar as an ICS
 *
 * Options :
 *   -u | -r => user login / resource id (mandatory)
 *   -d => user domain name (mandatory if user)
 *   -f => file to import (Only for import )
 *   -t => date for delete event, form example: 09/10/2009 (For export and reset )
 *   -m => Mode, default is interactive. Possible value : backup 
 *      in "Backup mode", store ICS into a file not into default
 *      output for all users (Only for export)
 *   -max => max imported users for fragmented exportation
 *   -p => page number for fragmented exportation
 */

HELP;
;
  }
  exit(constant($status_code_str));
}

$options = getopt('u:d:t:f:m:'.RESOURCE_OPTION.':'.MAX_OPTION.':'.PAGE_OPTION.':');
if(!$options || !is_array($options) || count($options) == 0) {
  exit_with_status('ERR_CONTEXT');
}
$opts = count($options);
foreach($options as $option => $value) {
  if($value === false) {
    exit_with_status('ERR_INVALID_ARGUMENT');
  }
}
if($argc != (($opts + 1)*2)) {
  exit_with_status('ERR_TO_MANY_ARGUMENT');
}

if ($obminclude == '') $obminclude = 'obminclude';
include("$obminclude/global.inc");
require($calendar_path.'calendar_query.inc');
require($calendar_path.'calendar_display.inc');
require("$obminclude/of/of_right.inc");
require_once("$obminclude/of/of_category.inc");

$cmd = $argv[($argc - 1)];

if($options['m'] != 'backup') {
  if ($cmd == 'export_resources') {
    $resources[] = array('id' => $options['r']);
    
  } else if (!empty($options['u']) || $cmd != 'import') {
    $obm = get_userinfo_by_ape($options['u'],$options['d']);
    if (!$obm) {
      exit_with_status('ERR_UNKNOWN_USER');
    }
    $users[]=$obm;
    
  }
} else {
  if ($cmd == 'export') {
    $users = get_all_users($options[MAX_OPTION], $options[PAGE_OPTION]);
    
  } else if ($cmd == 'export_resources') {
    $resources = get_all_resources($options[MAX_OPTION], $options[PAGE_OPTION]);
  
  }
}

if(isset($options['t'])) {
  $params['export'] = 'date';
  $params['delete_meeting'] = '0';
  $params['event_before_date'] = of_isodate_convert($options['t']); 
  $params['event_before_date'] = new Of_Date($params['event_before_date']); 
} else {
  $params['event_before_date'] = date('Y-m-d H:i:s');
}

switch ($cmd) {
  case 'reset' :
    run_query_calendar_reset($obm['uid'],$params);
    break;
  case 'import' :
    if(!is_readable($options['f'])) {
      exit_with_status('ERR_FILE_UNREADABLE');
    }
    $force = false;
    
    if (empty($options['u'])) {
      list($u, $d) = split('@', basename($options['f']));
      $d = preg_replace('/\\.ics$/', '', $d);
      $obm = get_userinfo_by_ape($u,$d);
    }
    
    define('RFC2445_CRLF',               "\r\n");
    define('RFC2445_WSP',                "\t ");
    define('RFC2445_FOLDED_LINE_LENGTH', 75);
    $params['ics_tmp'] = $options['f'];
    run_query_icalendar_insert($params, $force);
    break;
    
  case 'export' :
    foreach($users as $obm) {
      if ($obm == NOT_FINISHED_MARKER) {
        echo "There is more users to export.\n";
        exit_with_status('NOT_FINISHED_MARKER');
        
      } else {
        ob_start();
        dis_calendar_export_handle($params);
        $ICS = ob_get_clean();
        
        if($options['m'] == 'backup') {
          if (!empty($ICS))
          {
            $fname = BACKUPDIR.DIRECTORY_SEPARATOR.$obm['login'].'@'.$obm['domain_name'].'.ics';
            echo "Writing to file $fname\n";
            $handle = fopen($fname, 'w');
            fwrite($handle, $ICS);
            fclose($handle);
          }
        } else {
          echo $ICS;
        }
      }
    }
    break;
  case 'export_resources' :
    foreach($resources as $resource) {
      if ($obm == NOT_FINISHED_MARKER) {
        echo "There is more resources to export.\n";
        exit_with_status('NOT_FINISHED_MARKER');
        
      } else {
        ob_start();
        dis_calendar_export_handle($params, 'resource', $resource['id']);
        $ICS = ob_get_clean();
        
        if($options['m'] == 'backup') {
          if (!empty($ICS))
          {
            $resource['name'] = str_replace('/', '-', $resource['name']);
            $fname = BACKUPDIR.DIRECTORY_SEPARATOR.'resource_'.$resource['id'].'@'.$resource['domain_name'].'.ics';
            echo "Writing to file $fname\n";
            $handle = fopen($fname, 'w');
            fwrite($handle, $ICS);
            fclose($handle);
          }
        } else {
          echo $ICS;
        }
      }
    }
    break;
  default :
    exit_with_status('ERR_INVALID_COMMAND');
}
exit_with_status('OK');


function get_userinfo_by_ape($login,$domain) {
  $obm_q = new DB_OBM ;

  $query = "SELECT 
    userobm_id,
    userobm_login,
    domain_name,
    domain_alias,
    domain_id
    FROM UserObm
    LEFT JOIN Domain ON userobm_domain_id = domain_id
    WHERE userobm_login='$login'
    AND domain_name = '$domain'
    ORDER BY userobm_login";
  display_debug_msg($query, $cdg_sql, 'get_user_info()');
  $obm_q->query($query);
  $exist = $obm_q->next_record();
  if ($exist && is_array($obm_q->Record)) {
    foreach ($obm_q->Record as $key => $user) {
      $small_key = str_replace('userobm_', '', $key);
      $u[$small_key] = $user;
    }
    $u['uid'] = $u['id'];
  } else {
    return false;
  }
 return $u; 
}

function get_all_users($max_users = null, $page = null) {
  $obm_q = new DB_OBM ;
  
  if ($max_users > 0 && $page > 0) {
    $limit = " LIMIT ". ($max_users * ($page - 1)). ", ". ($max_users + 1);
  }
  
  $query = "SELECT 
    userobm_id,
    userobm_login,
    domain_name,
    domain_alias,
    domain_id
    FROM UserObm
    LEFT JOIN Domain ON userobm_domain_id = domain_id
    WHERE domain_id != 0
    ORDER BY userobm_login
    $limit
    ";
  
  display_debug_msg($query, $cdg_sql, 'get_all_users()');
  $obm_q->query($query);
  $users = false;
    
  while($obm_q->next_record()) {
    $u = array();
    
    foreach ($obm_q->Record as $key => $user) {
      $small_key = str_replace('userobm_', '', $key);
      $u[$small_key] = $user;
    }
    
    $u['uid'] = $u['id'];
    $users[] = $u;
  }
  
  if (count($users) > $max_users) {
    array_pop($users);
    $users[] = NOT_FINISHED_MARKER;
  }
  
  $obm_q->free();
  
  unset($obm_q);
  
  return $users; 
}


function get_all_resources($max_records = null, $page = null) {
  $obm_q = new DB_OBM ;
  
  if ($max_records > 0 && $page > 0) {
    $limit = " LIMIT ". ($max_records * ($page - 1)). ", ". ($max_records + 1);
  }
  
  $query = "SELECT 
    resource_id,
    resource_name,
    domain_name,
    domain_alias,
    domain_id
    FROM Resource
    LEFT JOIN Domain ON resource_domain_id = domain_id
    WHERE domain_id != 0
    ORDER BY resource_name
    $limit
    ";
    
  display_debug_msg($query, $cdg_sql, 'get_all_resources()');
  $obm_q->query($query);
  $records = false;
    
  while($obm_q->next_record()) {
    $r = array();
    
    foreach ($obm_q->Record as $key => $user) {
      $small_key = str_replace('resource_', '', $key);
      $r[$small_key] = $user;
    }
    
    $records[] = $r;
  }
  
  if (count($records) > $max_records) {
    array_pop($records);
    $records[] = NOT_FINISHED_MARKER;
  }
  
  $obm_q->free();
  
  unset($obm_q);
  
  return $records; 
}

?>
